package net.knarcraft.jarupdater.utility;

import net.knarcraft.jarupdater.Main;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;

/**
 * A utility class for the downloader
 */
public final class DownloaderUtil {

    private static String applicationWorkDirectory;

    /**
     * Gets the file name of the downloaded file from the URL
     *
     * @param url <p>The URL to parse</p>
     * @return <p>The file name at the end of the URL, or an empty string if not able to parse the URL</p>
     */
    public static String getFileNameFromURL(String url) {
        String[] urlParts = url.split("/");
        if (urlParts.length < 4) {
            throw new IllegalArgumentException("URL is m");
        }
        return urlParts[urlParts.length - 1];
    }

    /**
     * Checks if a string ends with an extension
     *
     * @param string <p>The string to test</p>
     * @return <p>True if the string seems to end with an extension</p>
     */
    public static boolean stringEndsWithExtension(String string) {
        return string.matches(".*\\.[a-zA-Z]{3}$");
    }

    /**
     * Runs an executable given its name
     *
     * @param executable <p>The executable to execute</p>
     * @throws IOException <p>If unable to start the process</p>
     */
    public static void runExecutable(String executable) throws IOException {
        ProcessBuilder builder;
        if (executable.endsWith(".jar")) {
            builder = new ProcessBuilder("java", "-jar", executable, "nogui");
        } else {
            builder = new ProcessBuilder(executable);
        }
        builder.redirectErrorStream(true);
        builder.start();
    }

    /**
     * Downloads a file from a website and replaces the target file
     *
     * @param path    <p>The full url of the file to download</p>
     * @param outfile <p>The file to save to</p>
     * @return <p>True if successful. False otherwise</p>
     */
    public static boolean downloadFile(String path, Path outfile) {
        try {
            URL url = new URL(path);
            InputStream in = url.openStream();
            Files.copy(in, outfile, StandardCopyOption.REPLACE_EXISTING);
            return true;
        } catch (IOException e) {
            return false;
        }
    }

    /**
     * Retrieves the directory the .jar file is running from
     *
     * @return <p>The directory the .jar file is running from</p>
     */
    public static String getApplicationWorkDirectory() {
        if (applicationWorkDirectory == null) {
            try {
                applicationWorkDirectory = String.valueOf(new File(Main.class.getProtectionDomain().getCodeSource()
                        .getLocation().toURI().getPath()).getParentFile());
            } catch (URISyntaxException e) {
                e.printStackTrace();
                System.exit(1);
            }
        }
        return applicationWorkDirectory;
    }

}
